import datetime
from axiom import store
from xmantissa import stats, webadmin
from epsilon import extime
from epsilon.scripts import benchmark

def main():
    s = store.Store("stats.axiom")
    sampler = stats.StatSampler(store=s)
    t = extime.Time()
    svc = stats.StatsService(store=s, currentMinuteBucket=100)
    for x in range(100):

        scope = stats.Statoscope("")
        scope._stuffs = {'bandwidth_http_down': 0, 'bandwidth_pop3_up': 0, 'bandwidth_sip_up': 0, 'Imaginary logins': 0, 'bandwidth_pop3-grabber_down': 0, 'SMTP logins': 0, 'actionExecuted': 0, 'bandwidth_pop3_down': 0, 'bandwidth_ssh_up': 0, 'cursor_execute_time': 0.15660834312438965, 'actionDuration': 0, 'bandwidth_smtps_up': 0, 'messages_grabbed': 0, 'bandwidth_smtps_down': 0, 'athena_messages_sent': 0, 'page_renders': 0, 'autocommits': 0, 'cache_hits': 7, 'bandwidth_telnet_up': 0, 'bandwidth_smtp_up': 0, 'bandwidth_sip_down': 0, 'bandwidth_http_up': 0, 'commits': 9, 'messagesSent': 0, 'bandwidth_ssh_down': 0, 'athena_messages_received': 0, 'bandwidth_https_up': 0, 'bandwidth_https_down': 0, 'cache_misses': 9, 'bandwidth_pop3s_down': 0, 'POP3 logins': 0, 'cursor_blocked_time': 0.0, 'messagesReceived': 0, 'bandwidth_telnet_down': 0, 'bandwidth_smtp_down': 0, 'mimePartsCreated': 0, 'bandwidth_pop3-grabber_up': 0, 'Web logins': 0, 'bandwidth_pop3s_up': 0}
        queryscope = stats.Statoscope("")
        queryscope._stuffs = {u'_axiom_query:SELECT main.item_axiom_batch__reliablelistener_v1.oid, main.item_axiom_batch__reliablelistener_v1.[backwardMark], main.item_axiom_batch__reliablelistener_v1.[forwardMark], main.item_axiom_batch__reliablelistener_v1.[lastRun], main.item_axiom_batch__reliablelistener_v1.[listener], main.item_axiom_batch__reliablelistener_v1.[processor], main.item_axiom_batch__reliablelistener_v1.[style] FROM main.item_axiom_batch__reliablelistener_v1 WHERE ((main.item_axiom_batch__reliablelistener_v1.[processor] = ?) AND (main.item_axiom_batch__reliablelistener_v1.[style] = ?) AND main.item_axiom_batch__reliablelistener_v1.[listener] NOT IN ()) ORDER BY main.item_axiom_batch__reliablelistener_v1.[lastRun] ASC ': 0, u'_axiom_query:SELECT main.item_xquotient_grabber_pop3uid_v1.[value] FROM main.item_xquotient_grabber_pop3uid_v1 WHERE (main.item_xquotient_grabber_pop3uid_v1.[grabberID] = ?)  ': 0, u'_axiom_query:SELECT main.item_axiom_powerup_connector_v1.oid, main.item_axiom_powerup_connector_v1.[interface], main.item_axiom_powerup_connector_v1.[item], main.item_axiom_powerup_connector_v1.[powerup], main.item_axiom_powerup_connector_v1.[priority] FROM main.item_axiom_powerup_connector_v1 WHERE ((main.item_axiom_powerup_connector_v1.[interface] = ?) AND (main.item_axiom_powerup_connector_v1.[item] = ?)) ORDER BY main.item_axiom_powerup_connector_v1.[priority] DESC ': 0, u'_axiom_query:SELECT main.item_xmantissa_stats_statsservice_v1.oid, main.item_xmantissa_stats_statsservice_v1.[currentMinuteBucket], main.item_xmantissa_stats_statsservice_v1.[currentQuarterHourBucket], main.item_xmantissa_stats_statsservice_v1.[installedOn] FROM main.item_xmantissa_stats_statsservice_v1   LIMIT 2': 0, u'_axiom_query:SELECT main.item_login_v2.oid, main.item_login_v2.[avatars], main.item_login_v2.[disabled], main.item_login_v2.[password] FROM main.item_login_v2, main.item_login_method_v2 WHERE ((main.item_login_method_v2.[domain] = ?) AND (main.item_login_method_v2.[localpart] = ?) AND (main.item_login_v2.[disabled] = ?) AND (main.item_login_method_v2.[account] = main.item_login_v2.oid))  ': 0, u'_axiom_query:SELECT main.item_login_method_v2.oid, main.item_login_method_v2.[account], main.item_login_method_v2.[domain], main.item_login_method_v2.[internal], main.item_login_method_v2.[localpart], main.item_login_method_v2.[protocol], main.item_login_method_v2.[verified] FROM main.item_login_method_v2   ': 0, u'_axiom_query:SELECT main.item_timed_event_v1.oid, main.item_timed_event_v1.[runnable], main.item_timed_event_v1.[time] FROM main.item_timed_event_v1 WHERE (main.item_timed_event_v1.[time] < ?) ORDER BY main.item_timed_event_v1.[time] ASC LIMIT 1': 0, u'_axiom_query:SELECT main.item_login_v2.oid, main.item_login_v2.[avatars], main.item_login_v2.[disabled], main.item_login_v2.[password] FROM main.item_login_v2   ': 0, u'_axiom_query:SELECT main.item_xmantissa_stats_statbucket_v1.oid, main.item_xmantissa_stats_statbucket_v1.[index], main.item_xmantissa_stats_statbucket_v1.[interval], main.item_xmantissa_stats_statbucket_v1.[time], main.item_xmantissa_stats_statbucket_v1.[type], main.item_xmantissa_stats_statbucket_v1.[value] FROM main.item_xmantissa_stats_statbucket_v1 WHERE ((main.item_xmantissa_stats_statbucket_v1.[type] = ?) AND (main.item_xmantissa_stats_statbucket_v1.[interval] = ?) AND ((main.item_xmantissa_stats_statbucket_v1.[index] >= ?) OR (main.item_xmantissa_stats_statbucket_v1.[index] <= ?)))  ': 0, u'_axiom_query:SELECT main.item_timed_event_v1.oid, main.item_timed_event_v1.[runnable], main.item_timed_event_v1.[time] FROM main.item_timed_event_v1  ORDER BY main.item_timed_event_v1.[time] ASC LIMIT 1': 0, u'_axiom_query:SELECT main.item_persistent_session_v1.oid, main.item_persistent_session_v1.[authenticatedAs], main.item_persistent_session_v1.[lastUsed], main.item_persistent_session_v1.[sessionKey] FROM main.item_persistent_session_v1 WHERE (main.item_persistent_session_v1.[sessionKey] = ?)  ': 0, u'_axiom_query:SELECT main.item_login_v2.oid, main.item_login_v2.[avatars], main.item_login_v2.[disabled], main.item_login_v2.[password] FROM main.item_login_v2 WHERE (main.item_login_v2.[disabled] != ?)  ': 0, u'_axiom_query:SELECT main.item_login_v2.oid, main.item_login_v2.[avatars], main.item_login_v2.[disabled], main.item_login_v2.[password] FROM main.item_login_v2, main.item_login_method_v2 WHERE ((main.item_login_method_v2.[domain] IS NULL) AND (main.item_login_method_v2.[localpart] = ?) AND (main.item_login_v2.[disabled] = ?) AND (main.item_login_method_v2.[account] = main.item_login_v2.oid))  ': 0, u'_axiom_query:SELECT main.item_mantissa_installed_offering_v1.oid, main.item_mantissa_installed_offering_v1.[application], main.item_mantissa_installed_offering_v1.[offeringName] FROM main.item_mantissa_installed_offering_v1 WHERE (main.item_mantissa_installed_offering_v1.[offeringName] = ?)  LIMIT 1': 0, u'_axiom_query:SELECT main.item_imaginary_wiring_realm_playercredentials_v1.oid, main.item_imaginary_wiring_realm_playercredentials_v1.[actor], main.item_imaginary_wiring_realm_playercredentials_v1.[password], main.item_imaginary_wiring_realm_playercredentials_v1.[username] FROM main.item_imaginary_wiring_realm_playercredentials_v1 WHERE (main.item_imaginary_wiring_realm_playercredentials_v1.[actor] = ?)  ': 0, u'_axiom_query:SELECT main.item_login_system_v1.oid, main.item_login_system_v1.[failedLogins], main.item_login_system_v1.[loginCount] FROM main.item_login_system_v1   ': 0, '_axiom_query:SELECT main.item_xmantissa_stats_statbucket_v1.oid, main.item_xmantissa_stats_statbucket_v1.[index], main.item_xmantissa_stats_statbucket_v1.[interval], main.item_xmantissa_stats_statbucket_v1.[time], main.item_xmantissa_stats_statbucket_v1.[type], main.item_xmantissa_stats_statbucket_v1.[value] FROM main.item_xmantissa_stats_statbucket_v1 WHERE ((main.item_xmantissa_stats_statbucket_v1.[interval] = ?) AND (main.item_xmantissa_stats_statbucket_v1.[type] = ?) AND (main.item_xmantissa_stats_statbucket_v1.[time] = ?))  ': 0, '_axiom_query:SELECT main.item_xmantissa_stats_statbucket_v1.oid, main.item_xmantissa_stats_statbucket_v1.[index], main.item_xmantissa_stats_statbucket_v1.[interval], main.item_xmantissa_stats_statbucket_v1.[time], main.item_xmantissa_stats_statbucket_v1.[type], main.item_xmantissa_stats_statbucket_v1.[value] FROM main.item_xmantissa_stats_statbucket_v1 WHERE ((main.item_xmantissa_stats_statbucket_v1.[interval] = ?) AND (main.item_xmantissa_stats_statbucket_v1.[type] = ?) AND (main.item_xmantissa_stats_statbucket_v1.[time] = ?))  ': 0, '_axiom_query:SELECT main.item_xmantissa_stats_statbucket_v1.oid, main.item_xmantissa_stats_statbucket_v1.[index], main.item_xmantissa_stats_statbucket_v1.[interval], main.item_xmantissa_stats_statbucket_v1.[time], main.item_xmantissa_stats_statbucket_v1.[type], main.item_xmantissa_stats_statbucket_v1.[value] FROM main.item_xmantissa_stats_statbucket_v1 WHERE ((main.item_xmantissa_stats_statbucket_v1.[index] = ?) AND (main.item_xmantissa_stats_statbucket_v1.[interval] = ?))  ': 0, u'_axiom_query:SELECT main.item_imaginary_objects_exit_v1.oid, main.item_imaginary_objects_exit_v1.[fromLocation], main.item_imaginary_objects_exit_v1.[name], main.item_imaginary_objects_exit_v1.[sibling], main.item_imaginary_objects_exit_v1.[toLocation] FROM main.item_imaginary_objects_exit_v1 WHERE (main.item_imaginary_objects_exit_v1.[toLocation] = ?)  ': 0, u'_axiom_query:SELECT main.item_mantissa_installed_offering_v1.[offeringName] FROM main.item_mantissa_installed_offering_v1   ': 0, u'_axiom_query:SELECT main.item_imaginary_objects_exit_v1.oid, main.item_imaginary_objects_exit_v1.[fromLocation], main.item_imaginary_objects_exit_v1.[name], main.item_imaginary_objects_exit_v1.[sibling], main.item_imaginary_objects_exit_v1.[toLocation] FROM main.item_imaginary_objects_exit_v1 WHERE (main.item_imaginary_objects_exit_v1.[fromLocation] = ?)  ': 0, u'_axiom_query:SELECT main.item_axiom_subscheduler_parent_hook_v1.oid, main.item_axiom_subscheduler_parent_hook_v1 main.item_xmantissa_search_searchresult_v1.[identifier], main.item_xmantissa_search_searchresult_v1.[indexedItem] FROM main.item_xmantissa_search_searchresult_v1 WHERE (main.item_xmantissa_search_searchresult_v1.[indexedItem] = ?)  ': 0, '_axiom_query:SELECT main.item_xmantissa_stats_statbucket_v1.oid, main.item_xmantissa_stats_statbucket_v1.[index], main.item_xmantissa_stats_statbucket_v1.[interval], main.item_xmantissa_stats_statbucket_v1.[time], main.item_xmantissa_stats_statbucket_v1.[type], main.item_xmantissa_stats_statbucket_v1.[value] FROM main.item_xmantissa_stats_statbucket_v1 WHERE ((main.item_xmantissa_stats_statbucket_v1.[index] = ?) AND (main.item_xmantissa_stats_statbucket_v1.[interval] = ?) AND (main.item_xmantissa_stats_statbucket_v1.[type] = ?))  ': 0, u'_axiom_query:SELECT main.item_developer_site_v1.oid, main.item_developer_site_v1.[administrators], main.item_developer_site_v1.[developers] FROM main.item_developer_site_v1   ': 0}
        sampler.service = svc
        sampler.doStatSample(s, scope, t, [])
        sampler.doStatSample(s, queryscope, t, [], bucketType=stats.QueryStatBucket)
        t = t + datetime.timedelta(minutes=1)

    asf = webadmin.AdminStatsFragment()
    asf._initializeObserver = lambda : None
    asf.svc = svc
    benchmark.start()
    asf.buildPie()
    benchmark.stop()
    
if __name__ == '__main__':
    main()



# Copyright (c) 2008 Divmod.  See LICENSE for details.

"""
Tests for I{axiomatic mantissa} and other functionality provided by
L{axiom.plugins.mantissacmd}.
"""

from twisted.trial.unittest import TestCase
from twisted.python.filepath import FilePath
from twisted.internet.ssl import Certificate

from axiom.store import Store
from axiom.plugins.mantissacmd import genSerial, Mantissa
from axiom.test.util import CommandStubMixin

from xmantissa.ixmantissa import IOfferingTechnician
from xmantissa.port import TCPPort, SSLPort
from xmantissa.web import SiteConfiguration
from xmantissa.terminal import SecureShellConfiguration
from xmantissa.plugins.baseoff import baseOffering


class MiscTestCase(TestCase):
    def test_genSerial(self):
        """
        Test that L{genSerial} returns valid unique serials.
        """
        s1 = genSerial()
        self.assertTrue(isinstance(s1, int), '%r must be an int' % (s1,))
        self.assertTrue(s1 >= 0, '%r must be positive' % (s1,))
        s2 = genSerial()
        self.assertNotEqual(s1, s2)



class CertificateTestCase(CommandStubMixin, TestCase):
    """
    Tests for the certificate generated by L{Mantissa}.
    """
    def _getCert(self):
        """
        Get the SSL certificate from an Axiom store directory.
        """
        certFile = FilePath(self.dbdir).child('files').child('server.pem')
        return Certificate.loadPEM(certFile.open('rb').read())


    def test_uniqueSerial(self):
        """
        Test that 'axiomatic mantissa' generates SSL certificates with a
        different unique serial on each invocation.
        """
        m = Mantissa()
        m.parent = self

        self.dbdir = self.mktemp()
        self.store = Store(self.dbdir)
        m.parseOptions(['--admin-password', 'foo'])
        cert1 = self._getCert()

        self.dbdir = self.mktemp()
        self.store = Store(self.dbdir)
        m.parseOptions(['--admin-password', 'foo'])
        cert2 = self._getCert()

        self.assertNotEqual(cert1.serialNumber(), cert2.serialNumber())


    def test_commonName(self):
        """
        C{axiomatic mantissa} generates an SSL certificate with the domain part
        of the admin username as its common name.
        """
        m = Mantissa()
        m.parent = self
        self.dbdir = self.mktemp()
        self.store = Store(filesdir=FilePath(self.dbdir).child("files").path)
        m.parseOptions(['--admin-user', 'admin@example.com', '--admin-password', 'foo'])
        cert = self._getCert()
        self.assertEqual(cert.getSubject().commonName, "example.com")



class MantissaCommandTests(TestCase, CommandStubMixin):
    """
    Tests for L{Mantissa}.
    """
    def setUp(self):
        """
        Create a store to use in tests.
        """
        self.filesdir = self.mktemp()
        self.siteStore = Store(filesdir=self.filesdir)


    def test_baseOffering(self):
        """
        L{Mantissa.installSite} installs the Mantissa base offering.
        """
        options = Mantissa()
        options.installSite(self.siteStore, u"example.com", u"", False)

        self.assertEqual(
            IOfferingTechnician(self.siteStore).getInstalledOfferingNames(),
            [baseOffering.name])


    def test_httpPorts(self):
        """
        L{Mantissa.installSite} creates a TCP port and an SSL port for the
        L{SiteConfiguration} which comes with the base offering it installs.
        """
        options = Mantissa()
        options.installSite(self.siteStore, u"example.com", u"", False)

        site = self.siteStore.findUnique(SiteConfiguration)
        tcps = list(self.siteStore.query(TCPPort, TCPPort.factory == site))
        ssls = list(self.siteStore.query(SSLPort, SSLPort.factory == site))

        self.assertEqual(len(tcps), 1)
        self.assertEqual(tcps[0].portNumber, 8080)
        self.assertEqual(len(ssls), 1)
        self.assertEqual(ssls[0].portNumber, 8443)
        self.assertNotEqual(ssls[0].certificatePath, None)


    def test_hostname(self):
        """
        L{Mantissa.installSite} sets the C{hostname} of the
        L{SiteConfiguration} to the domain name it is called with.
        """
        options = Mantissa()
        options.installSite(self.siteStore, u"example.net", u"", False)
        site = self.siteStore.findUnique(SiteConfiguration)
        self.assertEqual(site.hostname, u"example.net")


    def test_sshPorts(self):
        """
        L{Mantissa.installSite} creates a TCP port for the
        L{SecureShellConfiguration} which comes with the base offering it
        installs.
        """
        options = Mantissa()
        options.installSite(self.siteStore, u"example.com", u"", False)

        shell = self.siteStore.findUnique(SecureShellConfiguration)
        tcps = list(self.siteStore.query(TCPPort, TCPPort.factory == shell))

        self.assertEqual(len(tcps), 1)
        self.assertEqual(tcps[0].portNumber, 8022)



from twisted.trial.unittest import SkipTest
from axiom.test.historic import stubloader
from xmantissa.people import Mugshot, Person

class MugshotTestCase(stubloader.StubbedTest):
    def testUpgrade(self):
        try:
            from PIL import Image
        except ImportError:
            raise SkipTest('PIL is not available')

        m = self.store.findUnique(Mugshot)
        p = self.store.findUnique(Person)

        self.assertIdentical(m.person, p)
        self.assertEqual(p.name, 'Bob')

        img = Image.open(m.smallerBody.open())
        self.assertEqual(img.size, (m.smallerSize, m.smallerSize))




"""
Tests for L{xmantissa.liveform}.
"""

from xml.dom.minidom import parseString

from zope.interface import implements
from zope.interface.verify import verifyObject

from twisted.internet.defer import Deferred
from twisted.trial.unittest import TestCase

from epsilon.hotfix import require
require('twisted', 'trial_assertwarns')

from nevow.page import renderer
from nevow.tags import directive, div, span
from nevow.loaders import stan
from nevow.flat import flatten
from nevow.inevow import IQ
from nevow.athena import expose

from xmantissa.liveform import (
    FORM_INPUT, TEXT_INPUT, PASSWORD_INPUT, CHOICE_INPUT, Parameter,
    TextParameterView, PasswordParameterView, ChoiceParameter,
    ChoiceParameterView, Option, OptionView, LiveForm, ListParameter,
    ListChangeParameterView, ListChangeParameter,
    RepeatedLiveFormWrapper, _LIVEFORM_JS_CLASS, _SUBFORM_JS_CLASS, EditObject,
    FormParameter, FormParameterView, FormInputParameterView)

from xmantissa.webtheme import getLoader
from xmantissa.test.rendertools import TagTestingMixin, renderLiveFragment
from xmantissa.ixmantissa import IParameter, IParameterView


class StubView(object):
    """
    Behaviorless implementation of L{IParameterView} used where such an object
    is required by tests.
    """
    implements(IParameterView)

    patternName = 'text'

    def setDefaultTemplate(self, tag):
        """
        Ignore the default template tag.
        """



class ParameterTestsMixin:
    """
    Mixin defining various tests common to different parameter view objects.
    """
    def viewFactory(self, parameter):
        """
        Instantiate a view object for the given parameter.
        """
        raise NotImplementedError("%s did not implement viewFactory")


    def test_fromInputs(self):
        """
        The parameter should provide a C{fromInputs} method for LiveForm to poke.
        """
        self.assertTrue(
            hasattr(self.param, 'fromInputs'),
            "Parameter did not even have fromInputs method, let alone "
            "implement it correctly.  Override this test method and "
            "assert something meaningful.")


    def test_comparison(self):
        """
        Parameter view objects should compare equal to other view objects of
        the same type which wrap the same underlying parameter object.
        """
        self.assertTrue(self.viewFactory(self.param) == self.viewFactory(self.param))
        self.assertFalse(self.viewFactory(self.param) != self.viewFactory(self.param))
        self.assertFalse(self.viewFactory(self.param) == object())
        self.assertTrue(self.viewFactory(self.param) != object())


    def test_name(self):
        """
        The I{name} renderer of the view object should render the name of the
        L{Parameter} it wraps as a child of the tag it is given.
        """
        tag = div()
        renderedName = renderer.get(self.view, 'name')(None, tag)
        self.assertTag(tag, 'div', {}, [self.name])


    def test_label(self):
        """
        The I{label} renderer of the view object should render the label of
        the L{Parameter} it wraps as a child of the tag it is given.
        """
        tag = div()
        renderedLabel = renderer.get(self.view, 'label')(None, tag)
        self.assertTag(tag, 'div', {}, [self.label])


    def test_withoutLabel(self):
        """
        The I{label} renderer of the view object should do nothing if the
        wrapped L{Parameter} has no label.
        """
        tag = div()
        self.param.label = None
        renderedOptions = renderer.get(self.view, 'label')(None, tag)
        self.assertTag(renderedOptions, 'div', {}, [])


    def _defaultRenderTest(self, fragmentName):
        loader = getLoader(fragmentName)
        document = loader.load()
        patternName = self.view.patternName + '-input-container'
        pattern = IQ(document).onePattern(patternName)
        self.view.setDefaultTemplate(pattern)
        html = flatten(self.view)

        # If it parses, well, that's the best we can do, given an arbitrary
        # template.
        document = parseString(html)


    def test_renderWithDefault(self):
        """
        The parameter view should be renderable using the default template.
        """
        return self._defaultRenderTest('liveform')


    def test_renderWithCompact(self):
        """
        The parameter view should be renderable using the compact template.
        """
        return self._defaultRenderTest('liveform-compact')


    def test_clone(self):
        """
        The parameter should be cloneable.
        """
        newDefault = object()
        param = self.param.clone(newDefault)
        self.assertTrue(isinstance(param, self.param.__class__))
        self.assertIdentical(param.name, self.param.name)
        self.assertIdentical(param.label, self.param.label)
        self.assertIdentical(param.description, self.param.description)
        self.assertIdentical(param.coercer, self.param.coercer)
        self.assertIdentical(param.default, newDefault)
        self.assertIdentical(param.viewFactory, self.param.viewFactory)



class TextLikeParameterViewTestsMixin:
    """
    Mixin defining tests for parameter views which are simple text fields.
    """
    def type():
        def get(self):
            raise AttributeError("%s did not define the type attribute")
        return get,
    type = property(*type())

    name = u'param name'
    label = u'param label'
    coercer = lambda value: value
    description = u'param desc'
    default = u'param default value'


    def setUp(self):
        """
        Create a L{Parameter} and a L{TextParameterView} wrapped around it.
        """
        self.param = Parameter(
            self.name, self.type, self.coercer, self.label, self.description,
            self.default)
        self.view = self.viewFactory(self.param)


    def test_default(self):
        """
        L{TextParameterView.value} should render the default value of the
        L{Parameter} it wraps as a child of the tag it is given.
        """
        tag = div()
        renderedDefault = renderer.get(self.view, 'default')(None, tag)
        self.assertTag(tag, 'div', {}, [self.default])


    def test_withoutDefault(self):
        """
        L{TextParameterView.value} should leave the tag it is given unchanged
        if the L{Parameter} it wraps has a C{None} default.
        """
        tag = div()
        self.param.default = None
        renderedDefault = renderer.get(self.view, 'default')(None, tag)
        self.assertTag(tag, 'div', {}, [])


    def test_description(self):
        """
        L{TextParameterView.description} should render the description of the
        L{Parameter} it wraps as a child of the tag it is given.
        """
        tag = div()
        renderedDescription = renderer.get(self.view, 'description')(None, tag)
        self.assertTag(tag, 'div', {}, [self.description])


    def test_withoutDescription(self):
        """
        L{TextParameterView.description} should leave the tag it is given
        unchanged if the L{Parameter} it wraps has no description.
        """
        tag = div()
        self.param.description = None
        renderedDescription = renderer.get(self.view, 'description')(None, tag)
        self.assertTag(tag, 'div', {}, [])


    def test_renderCompletely(self):
        """
        L{TextParameterView} should be renderable in the usual Nevow manner.
        """
        self.view.docFactory = stan(div[
                div(render=directive('name')),
                div(render=directive('label')),
                div(render=directive('default')),
                div(render=directive('description'))])
        html = flatten(self.view)
        self.assertEqual(
            html,
            '<div><div>param name</div><div>param label</div>'
            '<div>param default value</div><div>param desc</div></div>')



class TextParameterViewTests(TextLikeParameterViewTestsMixin,
                             TestCase, ParameterTestsMixin,
                             TagTestingMixin):
    """
    Tests for the view generation code for C{TEXT_INPUT} L{Parameter}
    instances.
    """
    type = TEXT_INPUT
    viewFactory = TextParameterView



class PasswordParameterViewTests(TextLikeParameterViewTestsMixin,
                                 TestCase, ParameterTestsMixin,
                                 TagTestingMixin):
    """
    Tests for the view generation code for C{PASSWORD_INPUT} L{Parameter}
    instances.
    """
    type = PASSWORD_INPUT
    viewFactory = PasswordParameterView



class ChoiceParameterTests(TestCase, ParameterTestsMixin, TagTestingMixin):
    """
    Tests for the view generation code for C{CHOICE_INPUT} L{Parameter}
    instances.
    """
    viewFactory = ChoiceParameterView

    def setUp(self):
        """
        Create a L{Parameter} and a L{ChoiceParameterView} wrapped around it.
        """
        self.type = CHOICE_INPUT
        self.name = u'choice name'
        self.choices = [
            Option(u'description one', u'value one', False),
            Option(u'description two', u'value two', False)]
        self.label = u'choice label'
        self.description = u'choice description'
        self.multiple = False
        self.param = ChoiceParameter(
            self.name, self.choices, self.label, self.description,
            self.multiple)
        self.view = self.viewFactory(self.param)


    def test_multiple(self):
        """
        L{ChoiceParameterView.multiple} should render the multiple attribute on
        the tag it is passed if the wrapped L{ChoiceParameter} is a
        L{MULTI_CHOICE_INPUT}.
        """
        tag = div()
        self.param.multiple = True
        renderedSelect = renderer.get(self.view, 'multiple')(None, tag)
        self.assertTag(tag, 'div', {'multiple': 'multiple'}, [])


    def test_fromInputs(self):
        """
        L{ChoiceParameter.fromInputs} should extract the inputs directed at it
        and pass them on to the coerce function.
        """
        self.assertEqual(
            self.param.fromInputs({self.name: ['0']}),
            self.choices[0].value)


    def test_single(self):
        """
        L{ChoiceParameterView.multiple} should not render the multiple
        attribute on the tag it is passed if the wrapped L{ChoiceParameter} is
        a L{CHOICE_INPUT}.
        """
        tag = div()
        renderedSelect = renderer.get(self.view, 'multiple')(None, tag)
        self.assertTag(tag, 'div', {}, [])


    def test_options(self):
        """
        L{ChoiceParameterView.options} should load the I{option} pattern from
        the tag it is passed and add copies of it as children to the tag for
        all of the options passed to L{ChoiceParameterView.__init__}.
        """
        option = span(pattern='option')
        tag = div[option]
        renderedOptions = renderer.get(self.view, 'options')(None, tag)
        self.assertEqual(
            renderedOptions.children[1:],
            [OptionView(index, c, None)
             for (index, c)
             in enumerate(self.choices)])


    def test_description(self):
        """
        L{ChoiceParameterView.description} should add the description of the
        wrapped L{ChoiceParameter} to the tag it is passed.
        """
        tag = div()
        renderedOptions = renderer.get(self.view, 'description')(None, tag)
        self.assertTag(renderedOptions, 'div', {}, [self.description])


    def test_withoutDescription(self):
        """
        L{ChoiceParameterView.description} should do nothing if the wrapped
        L{ChoiceParameter} has no description.
        """
        tag = div()
        self.param.description = None
        renderedOptions = renderer.get(self.view, 'description')(None, tag)
        self.assertTag(renderedOptions, 'div', {}, [])


    def test_clone(self):
        """
        L{ChoiceParameter} instances should be cloneable.
        """
        newChoices = [object()]
        param = self.param.clone(newChoices)
        self.assertTrue(isinstance(param, self.param.__class__))
        self.assertIdentical(param.name, self.param.name)
        self.assertIdentical(param.label, self.param.label)
        self.assertIdentical(param.type, self.param.type)
        self.assertIdentical(param.description, self.param.description)
        self.assertIdentical(param.multiple, self.param.multiple)
        self.assertIdentical(param.choices, newChoices)
        self.assertIdentical(param.viewFactory, self.param.viewFactory)



class OptionTests(TestCase, TagTestingMixin):
    """
    Tests for the view generation code for a single choice, L{OptionView}.
    """
    simpleOptionTag = div[
            div(render=directive('description')),
            div(render=directive('value')),
            div(render=directive('index')),
            div(render=directive('selected'))]

    def setUp(self):
        """
        Create an L{Option} and an L{OptionView} wrapped around it.
        """
        self.description = u'option description'
        self.value = u'option value'
        self.selected = True
        self.option = Option(self.description, self.value, self.selected)
        self.index = 3
        self.view = OptionView(self.index, self.option, self.simpleOptionTag)


    def test_description(self):
        """
        L{OptionView.description} should add the description of the option it
        wraps as a child to the tag it is passed.
        """
        tag = div()
        renderedDescription = renderer.get(self.view, 'description')(None, tag)
        self.assertTag(renderedDescription, 'div', {}, [self.description])


    def test_value(self):
        """
        L{OptionView.value} should add the value of the option it wraps as a
        child to the tag it is passed.
        """
        tag = div()
        renderedValue = renderer.get(self.view, 'value')(None, tag)
        self.assertTag(renderedValue, 'div', {}, [self.value])


    def test_index(self):
        """
        L{OptionView.index} should add the index passed to
        L{OptionView.__init__} to the tag it is passed.
        """
        tag = div()
        renderedIndex = renderer.get(self.view, 'index')(None, tag)
        self.assertTag(renderedIndex, 'div', {}, [self.index])


    def test_selected(self):
        """
        L{OptionView.selected} should add a I{selected} attribute to the tag it
        is passed if the option it wraps is selected.
        """
        tag = div()
        renderedValue = renderer.get(self.view, 'selected')(None, tag)
        self.assertTag(renderedValue, 'div', {'selected': 'selected'}, [])


    def test_notSelected(self):
        """
        L{OptionView.selected} should not add a I{selected} attribute to the
        tag it is passed if the option it wraps is not selected.
        """
        self.option.selected = False
        tag = div()
        renderedValue = renderer.get(self.view, 'selected')(None, tag)
        self.assertTag(renderedValue, 'div', {}, [])


    def test_renderCompletely(self):
        """
        L{ChoiceParameterView} should be renderable in the usual Nevow manner.
        """
        html = flatten(self.view)
        self.assertEqual(
            html,
            '<div><div>option description</div><div>option value</div>'
            '<div>3</div><div selected="selected"></div></div>')



class ListParameterTests(TestCase):
    """
    Tests for L{ListParameter}.
    """
    def test_fromInputs(self):
        """
        L{ListParameter.fromInputs} should extract multiple values from the
        mapping it is passed and coerce each value, returning a Deferred which
        fires with a list of all of the coerced values.
        """
        name = u'list param'
        param = ListParameter(name, int, 2)
        result = param.fromInputs({name + u'_0': [u'1'],
                                   name + u'_1': [u'3']})
        result.addCallback(self.assertEqual, [1, 3])
        return result



class FormParameterTests(TestCase):
    """
    Tests for L{Parameter} created with a type of L{FORM_INPUT} and for
    L{FormParameter}.
    """
    def test_deprecated(self):
        """
        Creating a L{Parameter} with a type of L{FORM_INPUT} should emit a
        deprecation warning referring to L{FormParameter}.
        """
        self.assertWarns(
            DeprecationWarning,
            "Create a FormParameter, not a Parameter with type FORM_INPUT",
            __file__,
            lambda: Parameter(None, FORM_INPUT, None))


    def test_viewFactory(self):
        """
        L{FormParameter.viewFactory} should return a L{FormParameterView}
        wrapped around the parameter.
        """
        parameter = FormParameter(lambda **kw: None, LiveForm(None, []))
        view = parameter.viewFactory(parameter, None)
        self.assertTrue(isinstance(view, FormParameterView))
        self.assertIdentical(view.parameter, parameter)


    def test_provides(self):
        """
        L{FormParameter} should provide L{IParameter}.
        """
        parameter = FormParameter(u'name', None)
        self.assertTrue(IParameter.providedBy(parameter))
        self.assertTrue(verifyObject(IParameter, parameter))


    def test_fromInputs(self):
        """
        L{FormParameter.fromInputs} should extract the input value which
        corresponds to the parameter's name and pass it to the invoke method of
        the wrapped form.
        """
        value = '-13'
        invoked = {}
        param = u'foo'
        form = LiveForm(invoked.update, [Parameter(param, TEXT_INPUT, int)])
        name = u'name'
        parameter = FormParameter(name, form)
        parameter.fromInputs({name: [{param: [value]}]})
        self.assertEqual(invoked, {param: int(value)})


    def test_compact(self):
        """
        L{FormParameter.compact} should call compact on the wrapped form.
        """
        class FakeForm(object):
            isCompact = False
            def compact(self):
                self.isCompact = True

        form = FakeForm()
        parameter = FormParameter(None, form)
        parameter.compact()
        self.assertTrue(form.isCompact)



class FormParameterViewTests(TestCase):
    """
    Tests for L{FormParameterView}.
    """
    def test_provides(self):
        """
        L{FormParameterView} should provide L{IParameterView}.
        """
        self.assertTrue(IParameterView.providedBy(FormParameterView(None)))


    def test_inputs(self):
        """
        The I{input} renderer of L{FormParameterView} should add a subform from
        its wrapped form as a child to the tag it is called with.
        """
        form = LiveForm(None, [])
        name = u'bar'
        parameter = FormParameter(name, form)
        view = FormParameterView(parameter)
        tag = div()
        inputRenderer = renderer.get(view, 'input')
        tag = inputRenderer(None, div())
        self.assertEqual(tag.tagName, 'div')
        self.assertEqual(tag.attributes, {})
        self.assertEqual(tag.children, [form])
        self.assertIdentical(form.fragmentParent, view)
        self.assertEqual(form.subFormName, name)



class FormInputParameterViewTests(TestCase):
    """
    Tests for deprecated L{FormInputParameterView}.
    """
    def test_inputs(self):
        """
        The I{input} renderer of L{FormParameterView} should add a subform from
        its wrapped form as a child to the tag it is called with.
        """
        form = LiveForm(None, [])

        name = u'bar'
        type = FORM_INPUT
        form = LiveForm(None, [])
        parameter = Parameter(name, type, form)
        view = FormInputParameterView(parameter)
        tag = div()
        inputRenderer = renderer.get(view, 'input')
        tag = inputRenderer(None, div())
        self.assertEqual(tag.tagName, 'div')
        self.assertEqual(tag.attributes, {})
        self.assertEqual(tag.children, [form])
        self.assertIdentical(form.fragmentParent, view)
        self.assertEqual(form.subFormName, name)


class LiveFormTests(TestCase, TagTestingMixin):
    """
    Tests for the form generation code in L{LiveForm}.
    """
    # Minimal tag which can be used with the form renderer.  Classes are only
    # used to tell nodes apart in the tests.
    simpleLiveFormTag = div[
        span(pattern='text-input-container'),
        span(pattern='password-input-container'),
        span(pattern='form-input-container'),
        span(pattern='liveform', _class='liveform-container'),
        span(pattern='subform', _class='subform-container')]


    def test_compact(self):
        """
        L{LiveForm.compact} should replace the existing C{docFactory} with one
        for the I{compact} version of the live form template.
        """
        form = LiveForm(None, [])
        form.compact()
        self.assertTrue(form.docFactory.template.endswith('/liveform-compact.html'))


    def test_recursiveCompact(self):
        """
        L{LiveForm.compact} should also call C{compact} on all of its subforms.
        """
        class StubChild(object):
            compacted = False
            def compact(self):
                self.compacted = True
        child = StubChild()
        form = LiveForm(None, [Parameter('foo', FORM_INPUT, child),
                               Parameter('bar', TEXT_INPUT, int),
                               ListParameter('baz', None, 3),
                               ChoiceParameter('quux', [])])
        form.compact()
        self.assertTrue(child.compacted)


    def test_descriptionSlot(self):
        """
        L{LiveForm.form} should fill the I{description} slot on the tag it is
        passed with the description of the form.
        """
        description = u"the form description"
        formFragment = LiveForm(None, [], description)
        formTag = formFragment.form(None, self.simpleLiveFormTag)
        self.assertEqual(formTag.slotData['description'], description)


    def test_formSlotOuter(self):
        """
        When it is not nested inside another form, L{LiveForm.form} should fill
        the I{form} slot on the tag with the tag's I{liveform} pattern.
        """
        def submit(**kw):
            pass
        formFragment = LiveForm(submit, [])
        formTag = formFragment.form(None, self.simpleLiveFormTag)
        self.assertTag(
            formTag.slotData['form'], 'span', {'class': 'liveform-container'},
            [])


    def test_formSlotInner(self):
        """
        When it has a sub-form name, L{LiveForm.form} should fill the I{form}
        slot on the tag with the tag's I{subform} pattern.
        """
        def submit(**kw):
            pass
        formFragment = LiveForm(submit, [])
        formFragment.subFormName = 'test-subform'
        formTag = formFragment.form(None, self.simpleLiveFormTag)
        self.assertTag(
            formTag.slotData['form'], 'span', {'class': 'subform-container'},
            [])


    def test_noParameters(self):
        """
        When there are no parameters, L{LiveForm.form} should fill the
        I{inputs} slot on the tag it uses to fill the I{form} slot with an
        empty list.
        """
        def submit(**kw):
            pass
        formFragment = LiveForm(submit, [])
        formTag = formFragment.form(None, self.simpleLiveFormTag)
        self.assertEqual(formTag.slotData['form'].slotData['inputs'], [])


    def test_parameterViewOverride(self):
        """
        L{LiveForm.form} should use the C{view} attribute of parameter objects,
        if it is not C{None}, to fill the I{inputs} slot on the tag it uses to
        fill the I{form} slot.
        """
        def submit(**kw):
            pass

        name = u'param name'
        label = u'param label'
        type = TEXT_INPUT
        coercer = lambda value: value
        description = u'param desc'
        default = u'param default value'

        view = StubView()
        views = {}
        viewFactory = views.get
        param = Parameter(
            name, type, coercer, label, description, default, viewFactory)
        views[param] = view

        formFragment = LiveForm(submit, [param])
        formTag = formFragment.form(None, self.simpleLiveFormTag)
        self.assertEqual(
            formTag.slotData['form'].slotData['inputs'],
            [view])


    def test_individualTextParameter(self):
        """
        L{LiveForm.form} should fill the I{inputs} slot on the tag it uses to
        fill the I{form} slot with a list consisting of one
        L{TextParameterView} when the L{LiveForm} is created with one
        C{TEXT_INPUT} L{Parameter}.
        """
        def submit(**kw):
            pass

        name = u'param name'
        label = u'param label'
        type = TEXT_INPUT
        coercer = lambda value: value
        description = u'param desc'
        default = u'param default value'
        param = Parameter(
            name, type, coercer, label, description, default)

        formFragment = LiveForm(submit, [param])
        formTag = formFragment.form(None, self.simpleLiveFormTag)
        self.assertEqual(
            formTag.slotData['form'].slotData['inputs'],
            [TextParameterView(param)])


    def test_individualPasswordParameter(self):
        """
        L{LiveForm.form} should fill the I{inputs} slot of the tag it uses to
        fill the I{form} slot with a list consisting of one
        L{TextParameterView} when the L{LiveForm} is created with one
        C{PASSWORD_INPUT} L{Parameter}.
        """
        def submit(**kw):
            pass

        name = u'param name'
        label = u'param label'
        type = PASSWORD_INPUT
        coercer = lambda value: value
        description = u'param desc'
        default = u'param default value'
        param = Parameter(
            name, type, coercer, label, description, default)

        formFragment = LiveForm(submit, [param])
        formTag = formFragment.form(None, self.simpleLiveFormTag)
        self.assertEqual(
            formTag.slotData['form'].slotData['inputs'],
            [PasswordParameterView(param)])


    def test_individualFormParameter(self):
        """
        L{LiveForm.form} should fill the I{inputs} slot of the tag it uses to
        fill the I{form} slot with a list consisting of one
        L{FormParameterView} when the L{LiveForm} is created with one
        L{FormParameter}.
        """
        parameter = FormParameter(u'form param', LiveForm(None, []))
        formFragment = LiveForm(lambda **kw: None, [parameter])
        formTag = formFragment.form(None, self.simpleLiveFormTag)
        self.assertEqual(
            formTag.slotData['form'].slotData['inputs'],
            [FormParameterView(parameter)])


    def test_individualFormInputParameter(self):
        """
        L{LiveForm.form} should fill the I{inputs} slot of the tag it uses to
        fill the I{form} slot with a list consisting of one
        L{FormInputParameterView} when the L{LiveForm} is created with one
        C{FORM_INPUT} L{Parameter}.
        """
        def submit(**kw):
            pass

        name = u'param name'
        type = FORM_INPUT
        coercer = LiveForm(None, [])
        param = Parameter(name, type, coercer)

        formFragment = LiveForm(submit, [param])
        formTag = formFragment.form(None, self.simpleLiveFormTag)
        self.assertEqual(
            formTag.slotData['form'].slotData['inputs'],
            [FormInputParameterView(param)])


    def test_liveformTemplateStructuredCorrectly(self):
        """
        When a L{LiveForm} is rendered using the default template, the form
        contents should end up inside the I{form} tag.

        As I understand it, this is a necessary condition for the resulting
        html form to operate properly.  However, due to the complex behavior of
        the HTML (or even XHTML) DOM and the inscrutability of the various
        specific implementations of it, it is not entirely unlikely that my
        understanding is, in some way, flawed.  If you know better, and believe
        this test to be in error, supplying a superior test or simply deleting
        this one may not be out of the question. -exarkun
        """
        def submit(**kw):
            pass

        name = u'param name'
        label = u'param label'
        type = PASSWORD_INPUT
        coercer = lambda value: value
        description = u'param desc'
        default = u'param default value'
        param = Parameter(
            name, type, coercer, label, description, default)

        formFragment = LiveForm(submit, [param])
        html = renderLiveFragment(formFragment)
        document = parseString(html)
        forms = document.getElementsByTagName('form')
        self.assertEqual(len(forms), 1)
        inputs = forms[0].getElementsByTagName('input')
        self.assertTrue(len(inputs) >= 1)


    def test_liveFormJSClass(self):
        """
        Verify that the C{jsClass} attribute of L{LiveForm} is
        L{_LIVEFORM_JS_CLASS}.
        """
        self.assertEqual(LiveForm.jsClass, _LIVEFORM_JS_CLASS)


    def test_subFormJSClass(self):
        """
        Verify that the C{jsClass} attribute of the form returned from
        L{LiveForm.asSubForm} is L{_SUBFORM_JS_CLASS}.
        """
        liveForm = LiveForm(lambda **k: None, ())
        subForm = liveForm.asSubForm(u'subform')
        self.assertEqual(subForm.jsClass, _SUBFORM_JS_CLASS)


    def test_invoke(self):
        """
        L{LiveForm.invoke} should take the post dictionary it is passed, call
        the coercer for each of its parameters, take the output from each,
        whether it is available synchronously or as a Deferred result, and pass
        the aggregate to the callable the L{LiveForm} was instantiated with.  It
        should return a L{Deferred} which fires with the result of the callable
        when it is available.
        """
        arguments = {}
        submitResult = object()
        def submit(**args):
            arguments.update(args)
            return submitResult

        syncCoerces = []
        syncResult = object()
        def syncCoercer(value):
            syncCoerces.append(value)
            return syncResult
        sync = Parameter(u'sync', None, syncCoercer, None, None, None)

        asyncCoerces = []
        asyncResult = Deferred()
        def asyncCoercer(value):
            asyncCoerces.append(value)
            return asyncResult
        async = Parameter(u'async', None, asyncCoercer, None, None, None)

        form = LiveForm(submit, [sync, async])
        invokeResult = form.invoke({sync.name: [u'sync value'],
                                    async.name: [u'async value']})

        # Both of the coercers should have been called with their respective
        # values.
        self.assertEqual(syncCoerces, [u'sync value'])
        self.assertEqual(asyncCoerces, [u'async value'])

        # The overall form callable should not have been called yet, since a
        # Deferred is still outstanding.
        self.assertEqual(arguments, {})

        # This will be the result of the Deferred from asyncCoercer
        asyncObject = object()

        def cbInvoke(result):
            self.assertEqual(result, submitResult)
            self.assertEqual(arguments, {sync.name: syncResult,
                                         async.name: asyncObject})
        invokeResult.addCallback(cbInvoke)
        asyncResult.callback(asyncObject)
        return invokeResult


    def test_callingInvokes(self):
        """
        Calling a LiveForm should be the same as calling its invoke method. 
        This isn't a public API.
        """
        returnValue = object()
        calledWith = []
        def coercer(**params):
            calledWith.append(params)
            return returnValue

        form = LiveForm(
            coercer, [Parameter(u'name', TEXT_INPUT, unicode,
                                u'label', u'descr', u'default')])
        result = form({u'name': [u'value']})
        self.assertEqual(calledWith, [{u'name': u'value'}])
        result.addCallback(self.assertIdentical, returnValue)
        return result



class ListChangeParameterViewTestCase(TestCase):
    """
    Tests for L{ListChangeParameterView}.
    """
    def setUp(self):
        class TestableLiveForm(LiveForm):
            _isCompact = False
            def compact(self):
                self._isCompact = True
        self.innerParameters = [Parameter('foo', TEXT_INPUT, int)]
        self.parameter = ListChangeParameter(
            u'repeatableFoo', self.innerParameters, defaults=[], modelObjects=[])
        self.parameter.liveFormFactory = TestableLiveForm
        self.parameter.repeatedLiveFormWrapper = RepeatedLiveFormWrapper
        self.view = ListChangeParameterView(self.parameter)


    def test_patternName(self):
        """
        L{ListChangeParameterView} should use I{repeatable-form} as its
        C{patternName}
        """
        self.assertEqual(self.view.patternName, 'repeatable-form')


    def _doSubFormTest(self, subFormWrapper):
        """
        C{subFormWrapper} (which we expect to be the result of
        L{self.parameter.formFactory}, wrapped in
        L{self.parameter.repeatedLiveFormWrapper) should be a render-ready
        liveform that knows its a subform.
        """
        self.failUnless(
            isinstance(subFormWrapper, RepeatedLiveFormWrapper))
        self.assertIdentical(subFormWrapper.fragmentParent, self.view)
        subForm = subFormWrapper.liveForm
        self.assertEqual(self.innerParameters, subForm.parameters)
        self.assertEqual(subForm.subFormName, self.parameter.name)


    def test_formsRendererReturnsSubForm(self):
        """
        The C{forms} renderer of L{ListChangeParameterView} should render
        the liveform that was passed to the underlying parameter, as a
        subform.
        """
        (form,) = renderer.get(self.view, 'forms')(None, None)
        self._doSubFormTest(form)


    def test_repeatFormReturnsSubForm(self):
        """
        The C{repeatForm} exposed method of L{ListChangeParameterView}
        should return the liveform that was passed to the underlying
        parameter, as a subform.
        """
        self._doSubFormTest(expose.get(self.view, 'repeatForm')())


    def test_formsRendererCompact(self):
        """
        The C{forms} renderer of L{ListChangeParameterView} should call
        C{compact} on the form it returns, if the parameter it is wrapping had
        C{compact} called on it.
        """
        self.parameter.compact()
        (renderedForm,) = renderer.get(self.view, 'forms')(None, None)
        self.failUnless(renderedForm.liveForm._isCompact)


    def test_repeatFormCompact(self):
        """
        The C{repeatForm} exposed method of of L{ListChangeParameterView}
        should call C{compact} on the form it returns, if the parameter it is
        wrapping had C{compact} called on it.
        """
        self.parameter.compact()
        renderedForm = expose.get(self.view, 'repeatForm')()
        self.failUnless(renderedForm.liveForm._isCompact)


    def test_formsRendererNotCompact(self):
        """
        The C{forms} renderer of L{ListChangeParameterView} shouldn't call
        C{compact} on the form it returns, unless the parameter it is wrapping
        had C{compact} called on it.
        """
        (renderedForm,) = renderer.get(self.view, 'forms')(None, None)
        self.failIf(renderedForm.liveForm._isCompact)


    def test_repeatFormNotCompact(self):
        """
        The C{repeatForm} exposed method of L{ListChangeParameterView}
        shouldn't call C{compact} on the form it returns, unless the parameter
        it is wrapping had C{compact} called on it.
        """
        renderedForm = expose.get(self.view, 'repeatForm')()
        self.failIf(renderedForm.liveForm._isCompact)


    def test_repeaterRenderer(self):
        """
        The C{repeater} renderer of L{ListChangeParameterView} should
        return an instance of the C{repeater} pattern from its docFactory.
        """
        self.view.docFactory = stan(div(pattern='repeater', foo='bar'))
        renderedTag = renderer.get(self.view, 'repeater')(None, None)
        self.assertEqual(renderedTag.attributes['foo'], 'bar')



class ListChangeParameterTestCase(TestCase):
    """
    Tests for L{ListChangeParameter}.
    """
    _someParameters = (Parameter('foo', TEXT_INPUT, int),)

    def setUp(self):
        self.innerParameters = [Parameter('foo', TEXT_INPUT, int)]
        self.defaultValues = {u'foo': -56}
        self.defaultObject = object()

        self.parameter = ListChangeParameter(
            u'repeatableFoo', self.innerParameters,
            defaults=[self.defaultValues],
            modelObjects=[self.defaultObject])


    def getListChangeParameter(self, parameters, defaults):
        return ListChangeParameter(
            name=u'stateRepeatableFoo', parameters=parameters,
            defaults=defaults,
            modelObjects=[object() for i in range(len(defaults))])


    def test_asLiveForm(self):
        """
        L{ListChangeParameter.asLiveForm} should wrap forms in
        L{ListChangeParameter.liveFormWrapperFactory}.
        """
        parameter = self.getListChangeParameter(self._someParameters, [])
        parameter.repeatedLiveFormWrapper = RepeatedLiveFormWrapper

        liveFormWrapper = parameter.asLiveForm()
        self.failUnless(
            isinstance(liveFormWrapper, RepeatedLiveFormWrapper))
        self.assertTrue(liveFormWrapper.removable)
        liveForm = liveFormWrapper.liveForm
        self.failUnless(isinstance(liveForm, LiveForm))
        self.assertEqual(liveForm.subFormName, parameter.name)
        self.assertEqual(liveForm.parameters, self._someParameters)


    def test_getInitialLiveForms(self):
        """
        Same as L{test_asLiveForm}, but looks at the single liveform returned
        from L{ListChangeParameter.getInitialLiveForms} when the parameter
        was constructed with no defaults.
        """
        parameter = self.getListChangeParameter(self._someParameters, [])
        parameter.repeatedLiveFormWrapper = RepeatedLiveFormWrapper

        liveFormWrappers = parameter.getInitialLiveForms()
        self.assertEqual(len(liveFormWrappers), 1)

        liveFormWrapper = liveFormWrappers[0]
        self.failUnless(
            isinstance(liveFormWrapper, RepeatedLiveFormWrapper))
        self.assertFalse(liveFormWrapper.removable)
        liveForm = liveFormWrapper.liveForm
        self.failUnless(isinstance(liveForm, LiveForm))
        self.assertEqual(liveForm.subFormName, parameter.name)
        self.assertEqual(liveForm.parameters, self._someParameters)


    def test_getInitialLiveFormsDefaults(self):
        """
        Same as L{test_getInitialLiveForms}, but for the case where the
        parameter was constructed with default values.
        """
        defaults = [{'foo': 1}, {'foo': 3}]
        parameter = self.getListChangeParameter(self._someParameters, defaults)
        parameter.repeatedLiveFormWrapper = RepeatedLiveFormWrapper

        liveFormWrappers = parameter.getInitialLiveForms()
        self.assertEqual(len(liveFormWrappers), len(defaults))
        for (liveFormWrapper, default) in zip(liveFormWrappers, defaults):
            self.failUnless(
                isinstance(liveFormWrapper, RepeatedLiveFormWrapper))
            self.assertTrue(liveFormWrapper.removable)

            liveForm = liveFormWrapper.liveForm
            self.failUnless(isinstance(liveForm, LiveForm))
            self.assertEqual(liveForm.subFormName, parameter.name)
            self.assertEqual(len(liveForm.parameters), 1)

            # Matches up with self._someParameters, except the default should
            # be different.
            innerParameter = liveForm.parameters[0]
            self.assertEqual(innerParameter.name, 'foo')
            self.assertEqual(innerParameter.type, TEXT_INPUT)
            self.assertEqual(innerParameter.coercer, int)
            self.assertEqual(innerParameter.default, default['foo'])


    def test_identifierMapping(self):
        """
        L{ListChangeParameter} should be able to freely convert between
        python objects and the opaque identifiers generated from them.
        """
        defaultObject = object()
        identifier = self.parameter._idForObject(defaultObject)
        self.assertIdentical(
            self.parameter._objectFromID(identifier), defaultObject)


    def test_extractCreations(self):
        """
        L{RepeatableFormParameter._extractCreations} should return a list of
        two-tuples giving the identifiers of new objects being created and
        their associated uncoerced arguments.
        """
        key = u'key'

        (modificationIdentifier,) = self.parameter._idsToObjects.keys()
        modificationValue = u'edited value'

        creationIdentifier = self.parameter._newIdentifier()
        creationValue = u'created value'

        dataSets = [
            {self.parameter._IDENTIFIER_KEY: creationIdentifier,
             key: creationValue},
            {self.parameter._IDENTIFIER_KEY: modificationIdentifier,
             key: modificationValue}]

        creations = list(self.parameter._extractCreations(dataSets))
        self.assertEqual(creations, [(creationIdentifier, {key: creationValue})])


    def test_extractEdits(self):
        """
        L{RepeatableFormParameter._extractEdits} should return a list of
        two-tuples giving the identifiers of existing model objects which might
        be about to change and their associated uncoerced arguments.
        """
        key = u'key'

        creationIdentifier = self.parameter._newIdentifier()
        creationValue = u'created value'

        modificationIdentifier = self.parameter._idForObject(
            self.defaultObject)
        modificationValue = u'edited value'

        dataSets = [
            {self.parameter._IDENTIFIER_KEY: creationIdentifier,
             key: creationValue},
            {self.parameter._IDENTIFIER_KEY: modificationIdentifier,
             key: modificationValue}]

        edits = list(self.parameter._extractEdits(dataSets))
        self.assertEqual(
            edits, [(modificationIdentifier, {key: modificationValue})])


    def test_coerceAll(self):
        """
        L{RepeatableFormParameter._coerceAll} should take a list of two-tuples
        and return a L{Deferred} which is called back with a list of tuples
        where the first element of each tuple is the first element of a tuple
        from the input and the second element of each tuple is the result of
        the L{Deferred} returned by a call to
        L{RepeatableFormParameter._coerceSingleRepetition} with the second
        element of the same tuple from the input.  The ordering of the input
        and output lists should be the same.
        """
        firstInput = object()
        firstValue = u'1'
        secondInput = object()
        secondValue = u'2'
        inputs = [(firstInput, {u'foo': firstValue}),
                  (secondInput, {u'foo': secondValue})]

        coerceDeferred = self.parameter._coerceAll(inputs)
        coerceDeferred.addCallback(
            self.assertEqual,
            [(firstInput, {u'foo': int(firstValue)}),
             (secondInput, {u'foo': int(secondValue)})])
        return coerceDeferred


    def test_coercion(self):
        """
        L{ListChangeParameter._coerceSingleRepetition} should call the
        appropriate coercers from the repeatable form's parameters.
        """
        d = self.parameter._coerceSingleRepetition({u'foo': [u'-56']})
        d.addCallback(self.assertEqual, {u'foo': -56})
        return d


    def test_coercerCreate(self):
        """
        L{ListChangeParameter.coercer} should be able to figure out that a
        repetition is new if it is associated with an identifier generated by
        C{asLiveForm}.
        """
        parameter = ListChangeParameter(
            u'repeatableFoo', self.innerParameters,
            defaults=[],
            modelObjects=[])

        # get an id allocated to us
        liveFormWrapper = parameter.asLiveForm()
        coerceDeferred = parameter.coercer(
            [{u'foo': [u'-56'],
              parameter._IDENTIFIER_KEY: liveFormWrapper.identifier}])
        def cbCoerced(submission):
            self.assertEqual(submission.edit, [])
            self.assertEqual(submission.delete, [])
            self.assertEqual(len(submission.create), 1)
            self.assertEqual(submission.create[0].values, {u'foo': -56})
            CREATED_OBJECT = object()
            submission.create[0].setter(CREATED_OBJECT)
            self.assertIdentical(
                parameter._objectFromID(liveFormWrapper.identifier),
                CREATED_OBJECT)
        coerceDeferred.addCallback(cbCoerced)
        return coerceDeferred


    def test_coercerCreateNoChange(self):
        """
        L{ListChangeParameter.coercer} should be able to figure out when
        nothing has been done to a set of values created by a previous
        submission.
        """
        parameter = ListChangeParameter(
            u'repeatableFoo', self.innerParameters,
            defaults=[],
            modelObjects=[])

        # get an id allocated to us
        liveFormWrapper = parameter.asLiveForm()
        identifier = liveFormWrapper.identifier

        value = {u'foo': [u'-56'], parameter._IDENTIFIER_KEY: identifier}
        coerceDeferred = parameter.coercer([value.copy()])

        def cbFirstSubmit(firstSubmission):
            firstSubmission.create[0].setter(None)

            # Resubmit the same thing
            return parameter.coercer([value.copy()])

        def cbSecondSubmit(secondSubmission):
            self.assertEqual(secondSubmission.create, [])
            self.assertEqual(secondSubmission.edit, [])
            self.assertEqual(secondSubmission.delete, [])

        coerceDeferred.addCallback(cbFirstSubmit)
        coerceDeferred.addCallback(cbSecondSubmit)
        return coerceDeferred


    def test_coercerEdit(self):
        """
        L{ListChangeParameter.coercer} should be able to figure out that a
        repetition is an edit if its identifier corresponds to an entry in the
        list of defaults.
        """
        (identifier,) = self.parameter._idsToObjects.keys()

        editDeferred = self.parameter.coercer(
            [{u'foo': [u'-57'],
              self.parameter._IDENTIFIER_KEY: identifier}])

        def cbEdit(submission):
            self.assertEqual(submission.create, [])
            self.assertEqual(submission.edit,
                             [EditObject(self.defaultObject, {u'foo': -57})])
            self.assertEqual(submission.delete, [])

        editDeferred.addCallback(cbEdit)
        return editDeferred


    def test_repeatedCoercerEdit(self):
        """
        L{ListChangeParameter.coercer} should work correctly with respect
        to repeated edits.
        """
        (identifier,) = self.parameter._idsToObjects.keys()

        editDeferred = self.parameter.coercer(
            [{u'foo': [u'-57'], self.parameter._IDENTIFIER_KEY: identifier}])

        def cbEdited(ignored):
            # edit it back to the initial value
            return self.parameter.coercer(
                [{u'foo': [u'-56'], self.parameter._IDENTIFIER_KEY: identifier}])

        def cbRestored(submission):
            self.assertEqual(submission.create, [])
            self.assertEqual(submission.edit,
                             [EditObject(self.defaultObject, {u'foo': -56})])
            self.assertEqual(submission.delete, [])

        editDeferred.addCallback(cbEdited)
        editDeferred.addCallback(cbRestored)
        return editDeferred


    def test_coercerNoChange(self):
        """
        L{ListChangeParameter.coercer} shouldn't include a repetition
        anywhere in its result if it corresponds to a default and wasn't
        edited.
        """
        (identifier,) = self.parameter._idsToObjects.keys()

        unchangedDeferred = self.parameter.coercer(
            [{u'foo': [u'-56'],
              self.parameter._IDENTIFIER_KEY: identifier}])

        def cbUnchanged(submission):
            self.assertEqual(submission.create, [])
            self.assertEqual(submission.edit, [])
            self.assertEqual(submission.delete, [])

        unchangedDeferred.addCallback(cbUnchanged)
        return unchangedDeferred


    def test_repeatedCoercerNoChange(self):
        """
        Same as L{test_coercerNoChange}, but with multiple submissions that
        don't change anything.
        """
        (identifier,) = self.parameter._idsToObjects.keys()

        editDeferred = self.parameter.coercer(
            [{u'foo': [u'-56'],
              self.parameter._IDENTIFIER_KEY: identifier}])

        def cbEdited(ignored):
            # Same values - no edit occurs.
            return self.parameter.coercer(
                [{u'foo': [u'-56'],
                  self.parameter._IDENTIFIER_KEY: identifier}])

        def cbUnedited(submission):
            self.assertEqual(submission.create, [])
            self.assertEqual(submission.edit, [])
            self.assertEqual(submission.delete, [])

        editDeferred.addCallback(cbEdited)
        editDeferred.addCallback(cbUnedited)
        return editDeferred


    def test_coercerDelete(self):
        """
        L{ListChangeParameter.coercer} should be able to figure out that a
        default was deleted if it doesn't get a repetition with a
        corresponding identifier.
        """
        deleteDeferred = self.parameter.coercer([])

        def cbDeleted(submission):
            self.assertEqual(submission.create, [])
            self.assertEqual(submission.edit, [])
            self.assertEqual(submission.delete, [self.defaultObject])

        deleteDeferred.addCallback(cbDeleted)
        return deleteDeferred


    def test_repeatedCoercerDelete(self):
        """
        L{ListChangeParameter.coercer} should only report a deletion the
        first time that it doesn't see a particular value.
        """
        deleteDeferred = self.parameter.coercer([])

        def cbDeleted(ignored):
            return self.parameter.coercer([])

        def cbUnchanged(submission):
            self.assertEqual(submission.create, [])
            self.assertEqual(submission.edit, [])
            self.assertEqual(submission.delete, [])

        deleteDeferred.addCallback(cbDeleted)
        deleteDeferred.addCallback(cbUnchanged)
        return deleteDeferred


    def test_coercerDeleteUnsubmitted(self):
        """
        L{ListChangeParameter.coercer} should not report as deleted an
        internal marker objects when a form is repeated but the repetition is
        omitted from the submission.
        """
        (identifier,) = self.parameter._idsToObjects.keys()

        # Creates some new state inside the parameter (yea, ick, state).
        repetition = self.parameter.asLiveForm()
        unchangedDeferred = self.parameter.coercer([
            {u'foo': [u'-56'],
             self.parameter._IDENTIFIER_KEY: identifier}])

        def cbUnchanged(submission):
            self.assertEqual(submission.create, [])
            self.assertEqual(submission.edit, [])
            self.assertEqual(submission.delete, [])

        unchangedDeferred.addCallback(cbUnchanged)
        return unchangedDeferred


    def test_makeDefaultLiveForm(self):
        """
        L{ListChangeParameter._makeDefaultLiveForm} should make a live
        form that has been correctly wrapped and initialized.
        """
        liveFormWrapper = self.parameter._makeDefaultLiveForm(
            (self.parameter.defaults[0], 1234))
        self.failUnless(isinstance(
            liveFormWrapper, self.parameter.repeatedLiveFormWrapper))
        liveForm = liveFormWrapper.liveForm
        self.failUnless(isinstance(liveForm, LiveForm))
        self.assertEqual(
            len(liveForm.parameters), len(self.innerParameters))
        for parameter in liveForm.parameters:
            self.assertEqual(parameter.default, self.defaultValues[parameter.name])


    def test_makeADefaultLiveFormChoiceParameter(self):
        """
        Verify that the parameter-defaulting done by
        L{ListChangeParameter._makeDefaultLiveForm} works for
        L{ChoiceParameter} instances.
        """
        param = ListChangeParameter(
            u'',
            [ChoiceParameter(
                u'choice',
                [Option(u'opt 1', u'1', True),
                 Option(u'opt 2', u'2', False)],
                u'label!',
                u'description!',
                multiple=True)])
        liveFormWrapper = param._makeDefaultLiveForm(
            ({u'choice': [u'1', u'2']}, 1234))
        liveForm = liveFormWrapper.liveForm
        clonedParams = liveForm.parameters
        self.assertEqual(len(clonedParams), 1)
        clonedChoiceParam = clonedParams[0]
        self.assertTrue(
            isinstance(clonedChoiceParam, ChoiceParameter))
        self.assertEqual(clonedChoiceParam.name, u'choice')
        self.assertEqual(clonedChoiceParam.label, u'label!')
        self.assertEqual(
            clonedChoiceParam.description, u'description!')
        self.assertTrue(clonedChoiceParam.multiple)

        self.assertEqual(len(clonedChoiceParam.choices), 2)
        (c1, c2) = clonedChoiceParam.choices
        self.assertEqual(c1.description, u'opt 1')
        self.assertEqual(c1.value, u'1')
        self.assertEqual(c1.selected, True)
        self.assertEqual(c2.description, u'opt 2')
        self.assertEqual(c2.value, u'2')
        self.assertEqual(c2.selected, True)


    def test_asLiveFormIdentifier(self):
        """
        L{ListChangeParameter.asLiveForm} should allocate an identifier
        for the new liveform, pass it to the liveform wrapper and put the
        placeholder value L{ListChangeParameter._NO_OBJECT_MARKER} into
        the object mapping.
        """
        liveFormWrapper = self.parameter.asLiveForm()
        self.assertIn(liveFormWrapper.identifier, self.parameter._idsToObjects)
        self.assertIdentical(
            self.parameter._objectFromID(liveFormWrapper.identifier),
            self.parameter._NO_OBJECT_MARKER)


    def test_correctIdentifiersFromGetInitialLiveForms(self):
        """
        L{ListChangeParameter.getInitialLiveForms} should return a list of
        L{RepeatedLiveFormWrapper} instances with C{identifier} attributes
        which correspond to the identifiers associated with corresponding
        model objects in the L{ListChangeParameter}.
        """
        # XXX This should really have more than one model object to make sure
        # ordering is tested properly.
        forms = self.parameter.getInitialLiveForms()
        self.assertEqual(len(forms), 1)
        self.assertIdentical(
            self.parameter._objectFromID(forms[0].identifier),
            self.defaultObject)


    def test_fromInputs(self):
        """
        L{RepeatableFormParameter.fromInputs} should call the appropriate
        coercers from the repeatable form's parameters.
        """
        (modifyIdentifier,) = self.parameter._idsToObjects.keys()

        # Make a new object to be deleted
        deleteObject = object()
        deleteIdentifier = self.parameter._idForObject(deleteObject)

        createIdentifier = self.parameter._newIdentifier()

        result = self.parameter.fromInputs({
                self.parameter.name: [[
                        {self.parameter._IDENTIFIER_KEY: modifyIdentifier,
                         u'foo': [u'-57']},
                        {self.parameter._IDENTIFIER_KEY: createIdentifier,
                         u'foo': [u'18']}]]})
        def cbCoerced(changes):
            self.assertEqual(len(changes.create), 1)
            self.assertEqual(changes.create[0].values, {u'foo': 18})
            self.assertEqual(len(changes.edit), 1)

            self.assertIdentical(changes.edit[0].object, self.defaultObject)
            self.assertEqual(changes.edit[0].values, {u'foo': -57})

            self.assertEqual(changes.delete, [deleteObject])

        result.addCallback(cbCoerced)
        return result



class RepeatedLiveFormWrapperTestCase(TestCase):
    """
    Tests for L{RepeatedLiveFormWrapper}.
    """
    def test_removeLinkRenderer(self):
        """
        Verify that the I{removeLink} renderer of L{RepeatedLiveFormWrapper}
        only returns the tag if the C{removable} argument passed to the
        constructor is C{True}.
        """
        fragment = RepeatedLiveFormWrapper(None, None, removable=True)
        removeLinkRenderer = renderer.get(fragment, 'removeLink', None)
        tag = div()
        self.assertIdentical(removeLinkRenderer(None, tag), tag)
        fragment.removable = False
        self.assertEqual(removeLinkRenderer(None, tag), '')



# -*- test-case-name: xmantissa.test.test_search -*-

from __future__ import division

from zope.interface import implements

from twisted.internet import defer
from twisted.python import log, components

from nevow import inevow, athena, tags

from axiom import attributes, item
from axiom.upgrade import registerDeletionUpgrader

from xmantissa import ixmantissa


class SearchResult(item.Item):
    """
    A temporary, in-database object associated with a particular search (ie,
    one time that one guy typed in that one search phrase) and a single item
    which was found in that search.  These live in the database to make it easy
    to display and sort them, but they are deleted when they get kind of
    oldish.

    These are no longer used.  The upgrader to version 2 unconditionally
    deletes them.
    """
    schemaVersion = 2

    indexedItem = attributes.reference()

    identifier = attributes.integer()

registerDeletionUpgrader(SearchResult, 1, 2)



class SearchAggregator(item.Item):
    implements(ixmantissa.ISearchAggregator, ixmantissa.INavigableElement)

    powerupInterfaces = (ixmantissa.ISearchAggregator, ixmantissa.INavigableElement)
    schemaVersion = 1
    typeName = 'mantissa_search_aggregator'

    installedOn = attributes.reference()
    searches = attributes.integer(default=0)

    # INavigableElement
    def getTabs(self):
        return []


    # ISearchAggregator
    def providers(self):
        return list(self.store.powerupsFor(ixmantissa.ISearchProvider))


    def count(self, term):
        def countedHits(results):
            total = 0
            for (success, result) in results:
                if success:
                    total += result
                else:
                    log.err(result)
            return total

        return defer.DeferredList([
            provider.count(term)
            for provider
            in self.providers()], consumeErrors=True).addCallback(countedHits)


    def search(self, *a, **k):
        self.searches += 1

        d = defer.DeferredList([
            provider.search(*a, **k)
            for provider in self.providers()
            ], consumeErrors=True)

        def searchCompleted(results):
            allSearchResults = []
            for (success, result) in results:
                if success:
                    allSearchResults.append(result)
                else:
                    log.err(result)
            return allSearchResults
        d.addCallback(searchCompleted)

        return d



def parseSearchTerm(term):
    """
    Turn a string search query into a two-tuple of a search term and a
    dictionary of search keywords.
    """
    terms = []
    keywords = {}
    for word in term.split():
        if word.count(':') == 1:
            k, v = word.split(u':')
            if k and v:
                keywords[k] = v
            elif k or v:
                terms.append(k or v)
        else:
            terms.append(word)
    term = u' '.join(terms)
    if keywords:
        return term, keywords
    return term, None



class AggregateSearchResults(athena.LiveFragment):
    fragmentName = 'search'

    def __init__(self, aggregator):
        super(AggregateSearchResults, self).__init__()
        self.aggregator = aggregator


    def head(self):
        return None


    def render_search(self, ctx, data):
        req = inevow.IRequest(ctx)
        term = req.args.get('term', [None])[0]
        charset = req.args.get('_charset_')[0]
        if term is None:
            return ''
        try:
            term = term.decode(charset)
        except LookupError:
            log.err('Unable to decode search query encoded as %s.' % charset)
            return tags.div[
                "Your browser sent your search query in an encoding that we do not understand.",
                tags.br,
                "Please set your browser's character encoding to 'UTF-8' (under the View menu in Firefox)."]
        term, keywords = parseSearchTerm(term)
        d = self.aggregator.search(term, keywords)
        def gotSearchResultFragments(fragments):
            for f in fragments:
                f.setFragmentParent(self)
            return fragments
        d.addCallback(gotSearchResultFragments)
        return d

components.registerAdapter(AggregateSearchResults, SearchAggregator, ixmantissa.INavigableFragment)



"""
Nits for scrolltable region widget.
"""

from zope.interface import implements

from axiom.item import Item
from axiom import attributes
from axiom.store import Store

from nevow.livetrial.testcase import TestCase
from nevow.athena import expose

from xmantissa.ixmantissa import IWebTranslator
from xmantissa.scrolltable import ScrollingElement
from xmantissa.webtheme import getLoader

class FakeTranslator(object):
    """
    Translate webIDs deterministically for ease of testing.

    @ivar store: the axiom store to retrieve items from.
    """
    implements(IWebTranslator)

    def __init__(self, store):
        """
        Create a FakeTranslator from a given axiom store.
        """
        self.store = store


    def fromWebID(self, webID):
        """
        Load an item from a hashed storeID.
        """
        return self.store.getItemByID(int(webID.split('-')[-1]))


    def toWebID(self, item):
        """
        Convert an item into a string identifier by hashing its storeID.
        """
        return 'webID-' + str(item.storeID)



class SampleRowItem(Item):
    """
    A sample item to be used as rows in the tests.
    """
    value = attributes.integer()



class ScrollingElementTestCase(TestCase):
    """
    Nits for L{ScrollingElement}
    """
    jsClass = u'Mantissa.Test.TestRegionLive.ScrollingElementTestCase'

    def getScrollingElement(self, rowCount):
        """
        Get a L{ScrollingElement}
        """
        s = Store()
        for x in xrange(rowCount):
            SampleRowItem(value=(x + 1) * 50, store=s)
        scrollingElement = ScrollingElement(
            s, SampleRowItem, None, (SampleRowItem.value,), None, True,
            FakeTranslator(s))
        scrollingElement.setFragmentParent(self)
        scrollingElement.docFactory = getLoader(
            scrollingElement.fragmentName)
        return scrollingElement
    expose(getScrollingElement)



# -*- test-case-name: xmantissa.test.historic.test_pyLuceneIndexer4to5 -*-
# Copyright 2005 Divmod, Inc.  See LICENSE file for details

from axiom.test.historic.stubloader import saveStub

from xmantissa.test.historic.stub_pyLuceneIndexer3to4 import createDatabase as _createDatabase

def createDatabase(*a, **kw):
    return _createDatabase(*a, **kw)

if __name__ == '__main__':
    saveStub(createDatabase, 10568)



# Copyright 2008 Divmod, Inc. See LICENSE file for details

"""
Tests for AMPConfiguration's 1 to 2 upgrader.
"""

from axiom.test.historic import stubloader
from axiom.userbase import LoginSystem

from xmantissa.ampserver import AMPConfiguration
from xmantissa.ixmantissa import IOneTimePadGenerator


class AMPConfigurationUpgradeTests(stubloader.StubbedTest):
    """
    Tests for AMPConfiguration's 1 to 2 upgrader.
    """
    def test_attributeCopied(self):
        """
        L{AMPConfiguration.loginSystem}'s value should have been preserved.
        """
        self.assertIdentical(
            self.store.findUnique(AMPConfiguration).loginSystem,
            self.store.findUnique(LoginSystem))


    def test_poweredUp(self):
        """
        L{AMPConfiguration} should be powered-up as L{IOneTimePadGenerator}.
        """
        self.assertIdentical(
            IOneTimePadGenerator(self.store),
            self.store.findUnique(AMPConfiguration))



# Copyright 2006-2008 Divmod, Inc.
# See LICENSE file for details

"""
Tests for L{xmantissa.websession}.
"""

from twisted.trial.unittest import TestCase
from nevow.testutil import FakeRequest

from xmantissa.websession import PersistentSessionWrapper, usernameFromRequest


class TestUsernameFromRequest(TestCase):

    def test_domainUnspecified(self):
        """
        Test that L{usernameFromRequest} adds the value of host header to the
        username in the request if the username doesn't already specify a
        domain.
        """
        request = FakeRequest(headers={'host': 'divmod.com'})
        request.args = {'username': ['joe']}
        username = usernameFromRequest(request)
        self.assertEqual(username, 'joe@divmod.com')


    def test_domainSpecified(self):
        """
        Test that L{usernameFromRequest} returns the username in the request
        if that username specifies a domain.
        """
        request = FakeRequest(headers={'host': 'divmod.com'})
        request.args = {'username': ['joe@notdivmod.com']}
        username = usernameFromRequest(request)
        self.assertEqual(username, 'joe@notdivmod.com')



class TestPersistentSessionWrapper(TestCase):
    """
    Tests for L{PersistentSessionWrapper}.
    """
    def test_savorSessionCookie(self):
        """
        L{PersistentSessionWrapper.savorSessionCookie} adds a cookie with a
        large maximum age and a request-appropriate domain to the request.
        """
        request = FakeRequest(headers={'host': 'example.com'})
        resource = PersistentSessionWrapper(
            None, None, domains=['example.org', 'example.com'])

        resource.savorSessionCookie(request)
        self.assertEqual(
            request.cookies, {resource.cookieKey: request.getSession().uid})


    def _cookieTest(self, host, cookie, **kw):
        """
        Assert that a L{PersistentSessionWrapper} created with the given
        keyword arguments returns C{cookie} from its C{cookieDomainForRequest}
        method when passed a request with C{host} as the value for its I{Host}
        header.
        """
        request = FakeRequest(headers={'host': host})
        resource = PersistentSessionWrapper(None, None, **kw)
        self.assertEqual(resource.cookieDomainForRequest(request), cookie)


    def test_missingHostHeaderCookie(self):
        """
        L{PersistentSessionWrapper.cookieDomainForRequest} returns C{None} if
        no host header is present.
        """
        self._cookieTest(None, None)


    def test_noDomainsNoSubdomainsCookie(self):
        """
        L{PersistentSessionWrapper.cookieDomainForRequest} returns C{None} if
        no domain sequence is provided and subdomains are disabled.
        """
        self._cookieTest('example.com', None)


    def test_noDomainsSubdomainsCookie(self):
        """
        L{PersistentSessionWrapper.cookieDomainForRequest} returns the hostname
        from the request prefixed with C{"."} if no domain sequence is provided
        and subdomains are enabled.
        """
        self._cookieTest('example.com', '.example.com', enableSubdomains=True)


    def test_domainNotFoundNoSubdomainsCookie(self):
        """
        L{PersistentSessionWrapper.cookieDomainForRequest} returns the C{None}
        if the hostname from the request is not found in the supplied domain
        sequence and subdomains are disabled.
        """
        self._cookieTest('example.com', None, domains=['example.org'])


    def test_domainNotFoundSubdomainsCookie(self):
        """
        L{PersistentSessionWrapper.cookieDomainForRequest} returns the hostname
        from the request prefixed with C{"."} if the hostname from the request
        is not found in the supplied domain sequence and subdomains are
        enabled.
        """
        self._cookieTest('example.com', ".example.com", domains=['example.org'],
                         enableSubdomains=True)


    def test_domainFoundNoSubdomainsCookie(self):
        """
        L{PersistentSessionWrapper.cookieDomainForRequest} returns C{None} if
        the hostname from the request is found in the supplied domain sequence
        and subdomains are disabled.
        """
        self._cookieTest('example.com', None, domains=['example.com'])


    def test_domainFoundSubdomainsCookie(self):
        """
        L{PersistentSessionWrapper.cookieDomainForRequest} returns the hostname
        from the request prefixed with C{"."} if the hostname from the request
        is found in the supplied domain sequence and subdomains are enabled.
        """
        self._cookieTest('example.com', ".example.com",
                         domains=['example.com'], enableSubdomains=True)


    def test_subdomainFoundNoSubdomainsCookie(self):
        """
        L{PersistentSessionWrapper.cookieDomainForRequest} returns C{None} if
        the hostname from the request is a subdomain of one of the domains in
        the supplied domain sequence but subdomains are disabled.
        """
        self._cookieTest('alice.example.com', None, domains=['example.com'])


    def test_subdomainFoundSubdomainsCookie(self):
        """
        L{PersistentSessionWrapper.cookieDomainForRequest} returns the domain
        from the supplied domain sequence prefixed with C{"."} that the
        hostname from the request is found to be a subdomain of, if it is found
        to be a subdomain of any of them and subdomains are enabled.
        """
        self._cookieTest('alice.example.com', '.example.com',
                         domains=['example.com'], enableSubdomains=True)


    def test_explicitPortNumberCookie(self):
        """
        L{PersistentSessionWrapper.cookieDomainForRequest} disregards the port
        number in the request host.
        """
        self._cookieTest('alice.example.com:8080', '.example.com',
                         domains=['example.com'], enableSubdomains=True)



from axiom.test.historic import stubloader
from xmantissa.signup import PasswordReset

class PasswordResetTestCase(stubloader.StubbedTest):
    def testUpgrade(self):
        self.assertEqual(self.store.count(PasswordReset), 0)



from axiom.test.historic import stubloader
from xmantissa.webadmin import DeveloperApplication
from xmantissa.webapp import PrivateApplication

class DATestCase(stubloader.StubbedTest):
    def testUpgrade(self):
        """
        Ensure upgraded fields refer to correct items.
        """
        self.assertEqual(self.store.findUnique(DeveloperApplication).privateApplication,
                         self.store.findUnique(PrivateApplication))



from axiom.test.historic import stubloader
from xmantissa.signup import Ticket
from xmantissa.product import Product
from xmantissa.webadmin import AdministrativeBenefactor
class TicketTestCase(stubloader.StubbedTest):
    def testTicket1to2(self):
        """
        Make sure Ticket upgrades OK and has a Product corresponding
        to the old AdministrativeBenefactor.
        """
        t = self.store.findUnique(Ticket)
        self.failUnless(isinstance(t.product, Product))
        self.assertEqual(t.product.types,
                         AdministrativeBenefactor.powerupNames)
